"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Entity_1 = require("./../Entity");
var index_1 = require("./../index");
var Tool_1 = require("./../Tool");
/**
 * Generator of particles
 */
var Particles = (function (_super) {
    __extends(Particles, _super);
    /* LIFECYCLE */
    /**
     * @constructor
     */
    function Particles() {
        var _this = _super.call(this) || this;
        /**
         * Know if the particles is loaded and ready to emit
         */
        _this.loaded = false;
        _this.setProps({
            type: Tool_1.Enum.TYPE.NONE,
            config: null,
            images: [],
            duration: 0,
            autoRun: true
        });
        _this.emitter = new index_1.PIXIParticles.Emitter(_this.container, null, { emit: false });
        _this.signals.propChange.bind("images", _this.onConfigurationChange.bind(_this));
        _this.signals.update.add(_this.updateFollow.bind(_this));
        return _this;
    }
    Particles.prototype.initialize = function (props) {
        _super.prototype.initialize.call(this, props);
        this.onConfigurationChange();
    };
    /**
     * @override
     */
    Particles.prototype.kill = function () {
        _super.prototype.kill.call(this);
        if (this.emitter) {
            this.emitter.destroy();
        }
    };
    /**
     * @update
     * @override
     */
    Particles.prototype.update = function (tick) {
        _super.prototype.update.call(this, tick);
        if (this.emitter.emit && this.loaded) {
            this.emitter.resetPositionTracking();
            this.emitter.updateSpawnPos(this.props.x, this.props.y);
            this.emitter.update(tick);
        }
    };
    /* METHODS */
    /**
     * Update config property
     * @param config - The next config property
     */
    Particles.prototype.updateConfig = function (config) {
        if (config === void 0) { config = {}; }
        this.setProps({ config: config });
        this.onConfigurationChange();
        return this;
    };
    /**
     * Run the particles emitter
     */
    Particles.prototype.run = function () {
        if (this.props.duration) {
            this.timers.addTimer("duration", this.props.duration, this.kill.bind(this));
        }
        this.emitter.emit = true;
    };
    /**
     * Stop the current emitter
     */
    Particles.prototype.stop = function () {
        this.timers.removeTimer("duration");
        this.emitter.cleanup();
        this.emitter.emit = false;
    };
    /**
     * Know if this object is emitting
     */
    Particles.prototype.isRunning = function () {
        return this.emitter.emit;
    };
    /* EVENTS */
    /**
     * @override
     */
    Particles.prototype.updateContainerPosition = function () { };
    /**
     * When property "configuration" has changed
     * @access protected
     */
    Particles.prototype.onConfigurationChange = function () {
        var _this = this;
        Tool_1.Assets.getTexture([].concat(this.props.images), function (textures) {
            _this.emitter.cleanup();
            _this.emitter.init(textures, _this.props.config);
            _this.loaded = true;
            if (_this.props.autoRun) {
                _this.run();
            }
            else {
                _this.emitter.emit = false;
            }
        });
    };
    return Particles;
}(Entity_1.Entity));
exports.Particles = Particles;
